use std::path::{Path, PathBuf};

use anyhow::Context;
use cargo_metadata::camino::{Utf8Path, Utf8PathBuf};
use serde::{Deserialize, Serialize};

fn lock_step_default() -> bool {
    true
}

#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct RustWorkspaceOptions {
    #[serde(default = "lock_step_default")]
    pub lock_step: bool,
}

impl RustWorkspaceOptions {
    pub fn execute(&self, path: &Path, next_version: impl AsRef<str>) -> anyhow::Result<()> {
        let members = self.get_workspace(path)?;

        Ok(())
    }

    pub fn get_workspace(&self, path: &Path) -> anyhow::Result<Vec<cargo_metadata::Package>> {
        let entries = std::fs::read_dir(path)
            .context(anyhow::anyhow!("could not read dir: {}", path.display()))?
            .flatten()
            .filter(|p| p.file_name().to_str().unwrap() == "Cargo.toml")
            .collect::<Vec<_>>();

        let cargo_manifest_file = match entries.first() {
            Some(x) => x,
            None => anyhow::bail!("did not find any Cargo.toml in: {}", path.display()),
        };

        let mut cmd = cargo_metadata::MetadataCommand::new();

        let manifest = cmd
            .no_deps()
            .manifest_path(cargo_manifest_file.path())
            .exec()
            .context("could not parse manifest")?;

        let members = manifest.workspace_members.iter().collect::<Vec<_>>();
        let workspace_members = manifest
            .packages
            .into_iter()
            .filter(|p| members.contains(&&p.id))
            .map(|mut p| {
                p.manifest_path = abs_path(&p.manifest_path);

                for dependency in p.dependencies.iter_mut() {
                    dependency.path = dependency.path.take().map(|path| abs_path(&path))
                }

                p
            })
            .collect::<Vec<_>>();

        Ok(workspace_members)
    }
}

fn abs_path(path: &Utf8Path) -> Utf8PathBuf {
    match path.canonicalize_utf8() {
        Ok(path) => path,
        Err(e) => {
            tracing::debug!("failed to transform manifest_path into abs path: {}", path);
            path.to_path_buf()
        }
    }
}
